@@ -904,6 +904,9 @@ imapx_connect_sync (CamelService *service,
 			priv->mailbox_updated_handler_id = 0;
 		}
 
+		if (priv->connected_server != NULL)
+			camel_imapx_server_shutdown (priv->connected_server);
+
 		g_clear_object (&priv->connected_server);
 		priv->connected_server = g_object_ref (imapx_server);
 
@@ -992,6 +995,9 @@ imapx_disconnect_sync (CamelService *service,
 		priv->mailbox_updated_handler_id = 0;
 	}
 
+	if (priv->connected_server != NULL)
+		camel_imapx_server_shutdown (priv->connected_server);
+
 	g_clear_object (&priv->connected_server);
 	g_clear_object (&priv->connecting_server);
 
@@ -2229,30 +2235,6 @@ exit:
 	return success;
 }
 
-static gboolean
-imapx_store_noop_sync (CamelStore *store,
-                       GCancellable *cancellable,
-                       GError **error)
-{
-	CamelIMAPXStore *imapx_store;
-	CamelIMAPXServer *imapx_server;
-	gboolean success = TRUE;
-
-	/* If we're not connected then this truly is a no-op. */
-
-	imapx_store = CAMEL_IMAPX_STORE (store);
-	imapx_server = camel_imapx_store_ref_server (imapx_store, NULL);
-
-	if (imapx_server != NULL) {
-		success = camel_imapx_server_noop (
-			imapx_server, NULL, cancellable, error);
-	}
-
-	g_clear_object (&imapx_server);
-
-	return success;
-}
-
 static void
 imapx_migrate_to_user_cache_dir (CamelService *service)
 {
@@ -2514,7 +2496,6 @@ camel_imapx_store_class_init (CamelIMAPXStoreClass *class)
 	store_class->create_folder_sync = imapx_store_create_folder_sync;
 	store_class->delete_folder_sync = imapx_store_delete_folder_sync;
 	store_class->rename_folder_sync = imapx_store_rename_folder_sync;
-	store_class->noop_sync = imapx_store_noop_sync;
 
 	/* Inherited from CamelNetworkService. */
 	g_object_class_override_property (
